function AdapCalcNewtonBasis()
% Main routine for adaptive calculation of a Newton Basis
% based on greedy Power Function minimization
clear all
close all
global RBFtype;  % type of RBF see frbf.m
global RBFpar;   % parameter of RBF, see frbf.m
global RBFscale; % scale of RBF, see frbf.m
%
% We start with three choices of kernels. See frbf.m
%
RBFtype='g';  % Gauss kernel
RBFscale=2;   
RBFpar=0;
%
% RBFtype='mq';  % multiquadric kernel
% RBFscale=2;   
% RBFpar=-2; % only INVERSE multiquadrics are pos.def. 
%
% RBFtype='w'; % Wendland kernel
% RBFscale=10;  % scale for Wendland must be larger  
% RBFpar=2;     % the kernel will have smoothness 2*RBFpar
%
% -----------------------data sites-----------------------------
%
he=.05;   % h for evaluation grid
% We use a special routine for point generation
[X, ind, Xemesh, Yemesh]=cutcircle(he); % generates points
%
% The mesh for surface plotting is (Xemesh,Yemesh).
% The index set ind describes the indices of points in the domain.
% This is needed to plot zero values outside the domain.
% The set X is the one we want to work on, like in the paper.
% It is a matrix of dimension N times 2.
% We shall greedily select inceasing subsets.
%
N=length(ind) % number of points to work on
%
% Now the greedy adaptive Newton-based method
%
maxN=30;  % Maximal number of data sites to be finally used
PowFunTol=1.e-14; % tolerance for square (!) of power function
% we plot the full point set X
plot(X(:,1),X(:,2),'.')
% we reserve  a vector for plotting values 
plotvalues=zeros(length(Xemesh(:)),1);
% The matrix v will contain the Newton basis, like in the paper,
% each column holds the values of one function on the set X
V=zeros(N,maxN);
% The vector indselect will contain the indices of
% the adaptively selected points of X
indselect=zeros(maxN,1);
%
% We start the algorithm by calculating the maximum
% of K(x,x). Formally, we use a full vector z,
% like in the paper, and we shall update it.
%
z=frbf(0.0,0)*ones(N,1);
% we pick the maximum zmax and its index indzmax
[zmax indzmax]=max(abs(z));
% and count iterations
iter=1;
% we already have the index of the first point:
indselect(iter,1)=indzmax;
% the list zmaxlist of zmax values will later be plotted
zmaxlist=[zmax];
% we need the vector u of the paper, here for m=0 in the paper
u=kermat(X,X(indzmax,:));
% and we put its normalization into the first column of V:
V(:,iter)=u/sqrt(z(indzmax,1));
% For plotting, we insert these values into the plot vector, and plot the
% first Newton basis function
plotvalues(ind)=V(:,iter);
figure
surf(Xemesh,Yemesh,reshape(plotvalues,size(Xemesh))); % or contour or surfc 
title(sprintf('Newton basis function %3d\n',iter));
% end of plotting
% To prepare for the iteration, we need the w vector of the paper
w=V(:,iter).^2;
% Now we are ready for iteration. We have the first basis function
% already.
for iter=2:maxN
    % The square of the Power Function on X is always in z-w,
    % see the paper...
    % Thus we get the maximum and its index:
    [zmax indzmax]=max(z-w);
    % We stop if the Power Function is small enough:
    if zmax<PowFunTol
        iter=iter-1;
        break;
    end
    % We save the max value for later plotting:
    zmaxlist=[zmaxlist zmax];
    % Now we calculate the u vector in two steps:
    y=V(indzmax,1:iter)';
    u=kermat(X,X(indzmax,:))-V(:,1:iter)*y;
    % and insert its normalization into V, using
    % an identity proven in the paper: 
    V(:,iter)=u/sqrt((z(indzmax,1)-w(indzmax,1)));
    % We update w (and automatically the Power Function) 
    w=w+V(:,iter).^2;
    % and store our new point index:
    indselect(iter,1)=indzmax;
    % For plotting:
    plotvalues(ind)=V(:,iter);
    figure
    surf(Xemesh,Yemesh,reshape(plotvalues,size(Xemesh))); % or contour or surfc 
    title(sprintf('Newton basis function %3d\n',iter));
end
figure
semilogy(sqrt(zmaxlist))
title('Decay of Power Function') 
xlabel('Iterations')
figure
plot(X(:,1),X(:,2),'.',X(indselect(1:iter,1),1),X(indselect(1:iter,1),2),'ro')
axis equal
title(sprintf('Selected %3d points out of %d\n',iter, N))

function [Xe, ind, xm, ym]=cutcircle(he)
% generates regular points Xe in circle with lower left quadrangle cut out
% The index list ind indicates the positions in meshgrid (xm,ym) 
% when the meshgrid is written in a single point list (xee, yee)
dx=2; % space dimension
dy=dx;
% he=0.01; % 1 D stepsize for evaluation grid. 
% The grid can consist of very many points.
% he=0.01 gives N=40401 points to work on
[xm, ym]=meshgrid(-1:he:1,-1:he:1);
xee=xm(:);
yee=ym(:);
ind=find((xee.^2+yee.^2<=1)&((xee>=0)|(yee>=0)) );
msk=zeros(size(xee));
msk(ind)=1.0;
% figure
% plot(xee(ind),yee(ind),'.')
% title('Discretized domain')
% disp('Hit enter to proceed')
% pause
Xe=[xee(ind), yee(ind)];


function y =  frbf (s, k)   
% k-th derivative of standard rbf kernel in f form, i.e.
% as a function of s=r^2/2.
global RBFtype;   % 'g'=gaussian, 'mq' = multiquadric
global RBFpar;
switch lower(RBFtype)
    case ('g')   % 'g'=gaussian,
        par=RBFpar;
        if mod(k,2)==0
            y=exp(-s);
        else
            y=-exp(-s);
        end
    case ('mq') % 'mq' = multiquadric, inverse or not...
        fac=1;
        ord=k;
        par=RBFpar;
        while ord>0
            ord=ord-1;
            fac=fac*par;
            par=par-2;
        end
        y=fac*(1+2*s).^(par/2);    
    case ('p')  % powers 
        fac=1;
        ord=k;
        par=RBFpar;
        while ord>0
            ord=ord-1;
            fac=fac*par;
            par=par-2;
        end
        y=fac*(2*s+eps).^(par/2);  
    case ('tp')  % thin-plate 
        fac=1;
        ord=k;
        par=RBFpar;
        su=0;
        while ord>0
            ord=ord-1;
            if ord==k-1
                su=1;
            else
                su=su*par+fac;
            end           
            fac=fac*par;
            par=par-2;
        end
        y=(2*s+eps).^(par/2);
        y=fac*y.*log(2*s+eps)/2 +su*y;    
    case ('ms')  % Matern/Sobolev
        y=(-1)^k*besselk(RBFpar-k,sqrt(2*s+eps)).*(sqrt(2*s+eps)).^(RBFpar-k);
    case ('w') % Wendland functions.
        % we use only those which are pos. def. in dimension at most 3.
        [coeff, expon]=wendcoeff(3+2*k, RBFpar-k);
        r=sqrt(2*s);
        ind=find(r<=1);
        u=zeros(size(ind));
        sp=ones(size(ind));
        sloc=r(ind);
        for i=1:length(coeff)
            u=u+coeff(i)*sp;
            sp=sp.*sloc;   
        end
        u=u.*(1-sloc).^expon;
        y=zeros(size(s));
        y(ind)=(-1)^k*u;
    otherwise
        error('RBF type not implemented')
end

function mat=kermat(X, Y)
% creates kernel matrix
% for two point sets X and Y  
global RBFscale
[nx dx]=size(X);
[ny dy]=size(Y);
if dx~=dy
    error('Unequal space dimension for kermat arguments');
end
mat=frbf(distsqh(X,Y)/RBFscale^2,0);


function dst=distsqh(p, q)
% calculates a np*nq matrix of halved squares of point distances 
% with two point sets p and q of np and nq points each.
[np pdim]=size(p);
[nq qdim]=size(q);
if pdim~=qdim 
     error('point sets of unequal dimension')
end
if pdim==1
    dst=(repmat(p,1,nq)-repmat(q',np,1)).^2/2;
    return
end
dst=p*q';
cp=sum((p.*p)')/2; % squared norms of p points, as row, halved
cq=sum((q.*q)')/2; % squared norms of q points, as row, halved
dst=repmat(cp',1,nq)+repmat(cq,np,1)-dst; 


function [coeff, expon]=wendcoeff(d,k)
% calculates coefficients and exponent for polynomial 
% part p_{d,k}(r) of Wendland functions 
% phi_{d,k}(r)=p_{d,k}(r)*(1-r)^expon
expon=floor(d/2)+k+1;
coeff=zeros(k+1,1);
coeff(1,1)=1;
for n=0:k-1
    coeff(n+2,1)=coeff(n+1,1)/(n+expon+2);
    for j=n+1:-1:2
        coeff(j,1)=(j*coeff(j+1,1)+coeff(j-1,1))/(expon+j); 
    end
    expon=expon+1;
    coeff(1,1)=coeff(2,1)/expon;
end
